#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <newick.h>
#include <stringutils.h>
//======================================================================
//    Lutz
//======================================================================
int getMatchingBracketPosition (char *str) {

  int brackets = 1;
  int i = 0;

  if (str [i++] != '(') {

    printf ("getMatchingBracket: string doesn't start with (\n");
    exit (2);

  }

  while (str [i] != '\0' && brackets > 0) {

    if (str [i] == '(') brackets++;
    if (str [i] == ')') brackets--;
    
    i++;

  }

  if (str [i - 1] != ')') {

    printf ("getMatchingBracket: Wrong format!\n");
    exit (2);

  }

  return i - 1;

}  


int getNumberOfEdges (char *tmp) {

  int kommata = 0;
  int brackets = 0;
  int i = 0;

  while (tmp [i] != '\0') {

    if (tmp [i] == '(')
      brackets++;

    if (tmp [i] == ')')
      brackets--;

    if (tmp [i] == ',' && brackets == 0)
      kommata++;

    i++;

  }

  return kommata + 1;

}


int getLabelAndEdgeLength (char *str, char **label, float *edgelength) {

  int i = 0;
  char *end = NULL;
  int length = 0;

  if (str [i] == '\0')
    return 0;
    
  if (str [i] == ':')

    *edgelength = strtod (&str [i + 1], &end);
    
  else {

    *edgelength = -0.1;

    if (str [i] == '\'') {

      i = 0;

      do {
	i++;
	i += strcspn (&str [i], "\'");
      } while (str [i + 2] != '\'' && str [i + 1] != '\0');

      if (str [i + 1] == '\0') {

	printf ("getLabelAndEdgeLength: Unterminated quoted string\n");
	exit (2);

      }

      i = i + 2;

    } else {

      i = 0;

      while (str [i] != ':' && str [i] != ',' && str [i] != ')' && str [i] != '\0')
	i++;

      if (str [i] == '\0') {

	printf ("getLabelAndEdgeLength: Unterminated string\n");
	exit (2);

      }
    }

    *label = getSpace4String (i + 1);
    strncpy (*label, str, i);
    length = i;

    if (str [i] == ':')
      *edgelength = strtod (&str [i + 1], &end);

  }

  if (end)
    length += (end - &str[i]) / sizeof (char);

  return length;

}



nodeptr createNode (char *str) {

  nodeptr newnode = NULL;
  int brackets = 0;
  int kommata = 0;
  char *tmp = NULL, *tmp2 = NULL;
  char *lastbracket = NULL;
  char *label = NULL;
  float length = 0.0;
  int i = 0, j = 0, k = 0, slen;

  if (str == NULL || strlen (str) == 0)
    return NULL;

  if (str [0] != '(') {

    printf ("createNode: Wrong format!\n");
    exit (2);

  }

  i = getMatchingBracketPosition (str);

  if (str [strlen (str) - 1] != ')') { /* Der String endet nicht mit einer
					  Klammer, d.h. es existiert ein
					  interner Bezeichner und / oder die
					  Angabe einer Astlaenge. */
    slen = getLabelAndEdgeLength (&str [i + 1], &label, &length);

  }
 
  tmp = getSpace4String (i + 1);
  strncpy (tmp, &str [1], i);
  tmp [i] = '\0';

  newnode = (nodeptr) malloc (sizeof (node));

  if (newnode == NULL) {

    printf ("createNode: Out of memory\n");
    exit (1);

  }

  newnode->numofedges = getNumberOfEdges (tmp);

  newnode->edges = (nodeptr *) malloc (newnode->numofedges * sizeof (nodeptr));

  if (newnode->edges == NULL) {

    printf ("createNode: Out of memory\n");
    exit (1);

  }

  newnode->label = label;
  newnode->edgelength = length;

  i = 0;

  while (tmp [i] != '\0') {

    if (tmp [i] == '(') {  /* Neuer Teilbaum (TB) */

      k = getMatchingBracketPosition (&tmp [i]);

      /* Existiert interner Label und / oder Information zur Kantenlaenge
	 fuer den neuen TB? */

      k += getLabelAndEdgeLength (&tmp [i + k + 1], &label, &length) + 1;
      /* Achtung Aenderung: k + 1 bzw. k anstelle von k + 2 bzw. k + 1 */
      tmp2 = getSpace4String (k + 1);
      strncpy (tmp2, &tmp [i], k);
      tmp2 [k] = '\0';

      newnode->edges [j] = createNode (tmp2);
      
      if (newnode->edges [j])
	newnode->edges [j]->parent = newnode;

      j++;
      free (tmp2);

      i = i + k;
      i = i + strspn (&tmp [i], "),");

    } else {

      /* 
	 Es wurde kein neuer TB gefunden, statt dessen muss der Bezeichner
	 eines externen Knotens eingelesen werden.
      */

      slen = getLabelAndEdgeLength (&tmp [i], &label, &length);

      newnode->edges [j] = (nodeptr) malloc (sizeof (node));

      if (!newnode->edges [j]) {

	printf ("createNode: Out of memory\n");
	exit (1);

      }

      newnode->edges [j]->parent     = newnode;
      newnode->edges [j]->label	     = label;
      newnode->edges [j]->edgelength = length;
      newnode->edges [j]->numofedges = 0;
      newnode->edges [j]->edges		= NULL;

      i = i + slen + 1;
      j++;

    }

  }

  return newnode;

}


treelist readNewickTreeFile (FILE *fp, char unrooted) {

  int len, pos, offs, brackets, c;
  char cut, weiter = 1;
  char str [STRSIZE];
  treelist trees = NULL, newtree = NULL;

  while (fscanf (fp, "%[^;]", str) != EOF) {

    len = strlen (str);
    pos = 0; offs = 0;
    brackets = 0; c = 0;

    while (pos + offs < len) {

      while (pos + offs < len && isspace (str [pos + offs]))
	offs++;

/*        if (pos + offs >= len) { */

/*  	printf ("main: Unexpected end of string\n"); */
/*  	exit (2); */

/*        } */

      if (str [pos + offs] == '[') {

	char *match;

	match = strchr ((char *) &str [pos + offs], ']');

	if (match == NULL) {

	  printf ("main: Unexpected end of string: can't find ]\n");
	  exit (2);

	}

	offs += (match - (&str [pos + offs])) / sizeof (char) + 1; 

      }

      str [pos] = str [pos + offs];

      if (str [pos] == '(')
	brackets++;

      if (str [pos] == ')')
	brackets--;

      if (brackets == 1 && str [pos] == ',')
	c++;

      pos++;

    }

    str [pos] = '\0';

    if (brackets != 0) {

      printf ("main: Wrong format: Couldn't find )\n");
      exit (2);

    }

    pos = 0; offs = 0;
    brackets = 0;
    cut = 0; weiter = 1;

    if (c == 1 && unrooted == 1) {
      while (str [pos + offs] != '\0') {

	if (str [pos + offs] == '(') {

	  if (brackets == 1 && cut == 0) {
	    weiter = 0;
	    offs++;
	  }

	  brackets++;

	}

	if (str [pos + offs] == ')') {

	  if (brackets == 2 && cut == 0) {

	    offs++;
	    cut = 1;
	    weiter = 0;

	  }

	  brackets--;

	}

	str [pos] = str [pos + offs];

	if (weiter)
	  pos++;
	else
	  weiter = 1;
      }

      str [pos] = '\0';

    }

    newtree = (treelist) malloc (sizeof (pathelement));

    if (!newtree) {

      printf ("main: Out of memory\n");
      exit (1);

    }

    newtree->leave = createNode (str);
    newtree->next = trees;

    if (newtree->leave != NULL)
      trees = newtree;
    else
      free (newtree);

    fscanf (fp, ";");

    bzero (str, STRSIZE * sizeof (char));

  }

  return trees;

}


int writeNewickTree (FILE *fp, nodeptr tree) {

  int i;

  if (!tree)
    return 1;

  if (tree->numofedges == 0) {

    fprintf (fp, "%s", tree->label);

    if (tree->edgelength > 0.0)
      fprintf (fp, ":%f", tree->edgelength);

  } else {

    fprintf (fp, "(");

    for (i = 0; i < tree->numofedges; i++) {

      writeNewickTree (fp, tree->edges [i]);

      if (i < tree->numofedges - 1)
	fprintf (fp, ",");

    }

    fprintf (fp, ")");

    if (tree->label != NULL)
      fprintf (fp, "%s", tree->label);

    if (tree->edgelength > 0.0)
      fprintf (fp, ":%f", tree->edgelength);


  }

  return 0;

}

int printNewickTree (nodeptr tree) {

  int i;

  if (!tree)
    return 1;

  if (tree->numofedges == 0) {

    printf ("%s", tree->label);

    if (tree->edgelength > 0.0)
      printf (":%f", tree->edgelength);

  } else {

    printf ("(");

    for (i = 0; i < tree->numofedges; i++) {

      printNewickTree (tree->edges [i]);

      if (i < tree->numofedges - 1)
	printf (",");

    }

    printf (")");

    if (tree->label != NULL)
      printf ("%s", tree->label);

    if (tree->edgelength > 0.0)
      printf (":%f", tree->edgelength);


  }

  return 0;

}


int writeNewickTrees (FILE *fp, treelist trees) {

  treelist help = trees;

  while (help) {

    writeNewickTree (fp, help->leave);
    fprintf (fp, ";\n");

    help = help->next;

  }

  return 0;

}


//======================================================
//         Achim
//======================================================
