#include <stringtree.h>
#include <math.h>
#include <string.h>


int comp_age(const void * s1, const void * s2);     //Vergleichsfunktion um die Baumknoten nach Alter zu sortieren
int comp_strings(const void * s1, const void * s2); //Vergleichsunktion um die Baumknoten nach ihren Bezeichnern zu sortieren,
                                                    //zunaechst nach Laenge, dann alphabetisch
int comp_char(const void * s1, const void * s2);    //Ein String wird alphabetsich umsortiert
int enthalten_in(char * substring, char * string);  //Testfunktion die testet ob ein String (byteweise) in einem anderen enthalten ist
void entferne_substring(char * substring, char * string);


//Initialisierungsroutine:
//Diese Routine erwartet die Anzahl der Blaetter
//und fordert dann entsprechend viel Speicher fuer ALLE Baumknoten an
sTree::sTree (int i){
  int j;
  b = i;
  n = (2 * i) - 1;
  IntNode = (struct IntNode **) malloc ((n) * sizeof(struct IntNode *));
  
  if(IntNode == NULL){
    printf("Erfolglose Speicheranforderung in Initialisierung von STree:stringtree.h.\n");
    exit(1);
  }	
  for(j = 0; j < n; j++){
    IntNode[j] = (struct IntNode *) malloc (sizeof(struct IntNode));
    IntNode[j] -> node = (char *) malloc ((i + 1) * sizeof(char));
    
    if(IntNode[j] == NULL || IntNode[j] -> node == NULL){
      printf("Erfolglose Speicheranforderung in Initialisierung von sTree:stringtree.h.\n");
      exit(1);
    }
    IntNode[j] -> node[0] = '\0';
  }
  intervall = (struct IntervallDaten **) malloc ((b + 1) * sizeof(struct IntervallDaten *));
  for(j = 0; j <= b; j++){
    intervall[j] = (struct IntervallDaten *) malloc (sizeof(IntervallDaten));
    intervall[j] -> kanten = (struct IntNode **) malloc (j * sizeof(struct IntNode *));
  } 
}	

//Einlesen eines Baumes in Stringnotation
void sTree::ReadString(char * ST){
  char * pt1;
  char * pt2;
  struct IntNode * nodept;
  char * st;
  char tmp[32];
  int i;
  st = (char *) malloc ((strlen(ST) + 1) * sizeof(char));
  strcpy(st, ST);
  pt1 = st;
  i = 0;
  while(pt2 != NULL){
    nodept = IntNode[i];

    while(*pt1 == ' '){
      pt1++;
    }
    if((pt2 = strchr(pt1, ':')) != NULL){
      *pt2 = '\0';
      strcpy(nodept -> node, pt1);
      pt1 = pt2;
      pt1++;
      pt2 = strchr(pt1, ' ');
      if(pt2 != NULL){
        *pt2 = '\0';
      }
      tmp[0] = '\0';
      strcpy(tmp, pt1);
      if((pt1 = strchr(tmp, ',')) != NULL){
        *pt1 = '.';
      }
      if(strlen(tmp) == 0){
        nodept -> tod = -1;
      }else{
        nodept -> tod = atof(tmp);
      }
    }else{
      pt2 = strchr(pt1, ' ');
      if(pt2 != NULL){
        *pt2 = '\0';
      }
      strcpy(nodept -> node, pt1);
      nodept -> tod = -1;
    }
    if(pt2 != NULL){
      pt1 = pt2;
      pt1++;
    }
    i++;
  }
   free(st);
  //Nun sortieren wir alle Stringbezeichner alphabetisch
  for(i = 0; i < n; i++){
    qsort(IntNode[i] -> node, strlen(IntNode[i] -> node), sizeof(char), comp_char);
  }
  sTree::StringSort();
  //Nun tragen wird den Geburtstag der Kanten ein
  for(i = 0; i < n; i++){
    IntNode[i] -> geburt = sTree::ElternBestimmung(IntNode[i]) -> tod;
    //printf("TEST: %i: %s: %f - %f\n", i, IntNode[i] -> node, IntNode[i] -> geburt, IntNode[i] -> tod);
  }


  //Beschreiben der Intervalldaten
  sTree::WriteIntervallData();
  //Nun sortieren wir wieder nach Strings (Laenge und Alphabet)

  //Berechnung der gesamten Baumlaenge (ohne die ersten beiden Level)
  L = 0;
  for(i = 0; i< n; i++){
    L = L + IntNode[i] -> tod - IntNode[i] -> geburt;
  }
  //Nun tragen wir die Brueder ein
  for(i = 0; i < n; i++){
    IntNode[i] -> bruderkante = sTree::BruderBestimmung(IntNode[i]);
  }

}




//Einlesen eines Baumes in Stringnotation mit Raten
void sTree::ReadStringRates(char * ST){
  char * pt1;
  char * pt2;
  struct IntNode * nodept;
  char * st;
  char tmp[32];    //string der laenge 32 um kantenlaenge zwischenzuspeichern
  int i;

  st = (char *) malloc ((strlen(ST) + 1) * sizeof(char));
  strcpy(st, ST);
  pt1 = st;
  i = 0;
  while(pt2 != NULL){
    nodept = IntNode[i];

    while(*pt1 == ' '){       			//leerzeichen am zeilenanfang
      pt1++;
    }
    if((pt2 = strchr(pt1, ':')) != NULL){
      *pt2 = '\0';
      strcpy(nodept -> node, pt1);
      pt1 = pt2;
      pt1++; 
      
      //kantenlaenge
      pt2 = strchr(pt1, '#');			/////////////////////////////////////////NEU
      if(pt2 != NULL){
        *pt2 = '\0';
      }
      tmp[0] = '\0';
      strcpy(tmp, pt1);
      if((pt1 = strchr(tmp, ',')) != NULL){	//dezimalkomma durch dezimalpkt. ersetzen
        *pt1 = '.';
      }
      if(strlen(tmp) == 0){
        nodept -> tod = -1;
      }
      else{
        nodept -> tod = atof(tmp);		//zeichenkette in double umwandeln 
      }
      pt1 = pt2;
      pt1++;
      
      //rate//////////////////////////////////////////NEU
      pt2 = strchr(pt1, ' ');
      if(pt2 != NULL){
        *pt2 = '\0';
      }
      tmp[0] = '\0';
      strcpy(tmp, pt1);
      if((pt1 = strchr(tmp, ',')) != NULL){	//dezimalkomma durch dezimalpkt ersetzen
        *pt1 = '.';
      }
      
      nodept -> rate = atof(tmp);
      if (nodept -> rate >= 0){     		////////////////////////////////////////NEU
        pt1 = pt2;
        pt1++;
      }
      else{
        printf("HGT Rate < 0 gefunden! Es sind nur positive HGT Raten erlaubt!\n\n");
	exit(1);
      }	
          
    }
    else{					//wenn keine raten UND keine laengen angegeben werden
      pt2 = strchr(pt1, ' ');
      if(pt2 != NULL){
        *pt2 = '\0';
      }
      strcpy(nodept -> node, pt1);
      nodept -> tod = -1;
    }
    if(pt2 != NULL){
      pt1 = pt2;
      pt1++;
    }
    i++;
  }
   free(st);
   
  //Nun sortieren wir alle Stringbezeichner alphabetisch jeweils innerhalb eines jeden knotens
  for(i = 0; i < n; i++){
    qsort(IntNode[i] -> node, strlen(IntNode[i] -> node), sizeof(char), comp_char);
  }
  
  sTree::StringSort();
  
  //Nun tragen wird den Geburtstag der Kanten ein und die rechte grenze im baumgroessen- und ratenbaumgroessenintervall ein
  //DIE INTERVALLE WERDEN IN STRINGSORT SORTIERUNG GESPEICHERT
  for(i = 0; i < n; i++){
    IntNode[i] -> geburt = sTree::ElternBestimmung(IntNode[i]) -> tod;
    if (i==0){
      IntNode[i] -> rechts_baumgroessenintervall = IntNode[i] -> tod - IntNode[i] -> geburt;
      IntNode[i] -> rechts_ratenbaumgroessenintervall = (IntNode[i] -> tod - IntNode[i] -> geburt) * IntNode[i] -> rate;
    }
    else{
      IntNode[i] -> rechts_baumgroessenintervall = IntNode[i-1] -> rechts_baumgroessenintervall + IntNode[i] -> tod - IntNode[i] -> geburt;
      IntNode[i] -> rechts_ratenbaumgroessenintervall = IntNode[i-1] -> rechts_ratenbaumgroessenintervall + (IntNode[i] -> tod - IntNode[i] -> geburt) * IntNode[i] -> rate;
    }  
    //printf("TEST: %i: %s: %f - %f\n", i, IntNode[i] -> node, IntNode[i] -> geburt, IntNode[i] -> tod);
    //printf("TEST: %i: %f: %f \n", i, IntNode[i] -> rechts_baumgroessenintervall, IntNode[i] -> rechts_ratenbaumgroessenintervall);
  }

  //Beschreiben der Intervalldaten
  sTree::WriteIntervallData();

  //Berechnung der gesamten Baumlaenge (ohne die ersten beiden Level)
  L = 0;
  for(i = 0; i< n; i++){
    L = L + IntNode[i] -> tod - IntNode[i] -> geburt;
  }
  //Nun tragen wir die Brueder ein
  for(i = 0; i < n; i++){
    IntNode[i] -> bruderkante = sTree::BruderBestimmung(IntNode[i]);
  }
}


//Sortieren der Knoten nach ihrem Alter
void sTree::AgeSort(){
  qsort(IntNode, n,  sizeof(struct IntNode *), comp_age);
  }
//Sortieren der Knoten nach ihren Bezeichnern (zuerst nach Laenge dann alphabetisch)
void sTree::StringSort(){
  qsort(IntNode, n, sizeof(struct IntNode *), comp_strings);
}

//Haengt Kante j in Kante i ohne Zeiten zu aendern  (Transfer von i nach j)
//Zurueckgegeben wird:
//0 falls nicht umgehaengt werden musste,
//1 falls umgeangt wurde aber die (ungewurzelte)
//  Topologie nicht veraendert wurde
//2 falls umgaengt und neue (auch ungewurzelte) Topologie
  int sTree::HaengeZeitlosUm(struct IntNode * nodept1, struct IntNode * nodept2){
    int index;
    struct IntNode * NODEPT1;
    struct IntNode * NODEPT2;
    //struct IntNode * zwei_generationen;
    int i_stat;
    int j_stat;
     int rueckgabe;
    //Zunaechst testen wir ob ueberhaupt umgehaehgt werden muss (nur wenn die
    //Elternknoten nicht identisch sind)
    NODEPT1 = sTree::ElternBestimmung(nodept1);
    NODEPT2 = sTree::ElternBestimmung(nodept2);

    if(NODEPT1 == NODEPT2){
      //printf("Es muss nicht umgehaengt werden.\n");
      return 0;
    }
    else{      //Es wird umgehaengt
    if( (NODEPT1 == IntNode[0] && sTree::ElternBestimmung(NODEPT2) == IntNode[0]) ||
        (NODEPT2 == IntNode[0] && sTree::ElternBestimmung(NODEPT1) == IntNode[0])){
        rueckgabe = 1;
    }
    else{
       rueckgabe = 2;
    }

    for(index = 0; index < n; index++){
      j_stat = enthalten_in( nodept2 -> node, IntNode[index] -> node );
      i_stat = enthalten_in( nodept1 -> node, IntNode[index] -> node );

      if(j_stat == 1 && i_stat == -1){
        entferne_substring(nodept2 -> node, IntNode[index] -> node);
      }
      else if( j_stat == -1 && i_stat == 1){
        strcat(IntNode[index] -> node, nodept2 -> node);
        qsort(IntNode[index] -> node, strlen(IntNode[index] -> node), sizeof(char), comp_char);
      }
    }
    //Beschriftung des neu entstandenen Knoten
    strcpy(NODEPT2 -> node, nodept1 -> node);
    strcat(NODEPT2 -> node, nodept2 -> node);

    qsort(NODEPT2 -> node, strlen(NODEPT2 -> node), sizeof(char), comp_char);
  }
  return rueckgabe;
}

//Haengt Kante j in Kante i mit Zeitpunkt zu aendern  (Transfer von i nach j)
//Zurueckgegeben wird:
// 0 falls nicht umgehaengt werden musste,
// 1 falls umgeangt wurde aber die (ungewurzelte)
//   Topologie nicht veraendert wurde
// 2 falls umgaengt und neue (auch ungewurzelte) Topologie
int sTree::HaengeZeitigUm(struct IntNode * nodept1, struct IntNode * nodept2, double time){
  int index;
  struct IntNode * NODEPT1;
  struct IntNode * NODEPT2;
  //struct IntNode * zwei_generationen;
  int i_stat;
  int j_stat;
  int rueckgabe;
  //printf("%s -> %s", nodept1 -> node, nodept2 -> node);     //---------------------------------------------------------------<DEBUG
  //printf("\n");
  //Zunaechst testen wir ob ueberhaupt umgehaehgt werden muss (nur wenn die Elternknoten nicht identisch sind)
  NODEPT1 = sTree::ElternBestimmung(nodept1);
  NODEPT2 = sTree::ElternBestimmung(nodept2);

  if(NODEPT1 == NODEPT2){
    //printf("Es muss nicht umgehaengt werden. Aber der Zeitpunkt wird eingetragen\n");
    NODEPT1 -> geburt = time;
    //printf("\n");//----------------------------------------------------------------------------------------------------<DEBUG
    return 0;
  }
  else{      //Es wird umgehaengt
    if( (NODEPT1 == IntNode[0] && sTree::ElternBestimmung(NODEPT2) == IntNode[0]) ||
	(NODEPT2 == IntNode[0] && sTree::ElternBestimmung(NODEPT1) == IntNode[0])){
      //rueckgabe = 1;
      //printf("\n");//----------------------------------------------------------------------------------------------------<DEBUG
      return 1;  //Doch nicht umhaengen in diesem Fall !!!!!
    }
    else{
      rueckgabe = 2;
    }
    
    for(index = 0; index < n; index++){
      j_stat = enthalten_in( nodept2 -> node, IntNode[index] -> node );
      i_stat = enthalten_in( nodept1 -> node, IntNode[index] -> node );
      
      if(j_stat == 1 && i_stat == -1){
	entferne_substring(nodept2 -> node, IntNode[index] -> node);
      }
      else if( j_stat == -1 && i_stat == 1){
	strcat(IntNode[index] -> node, nodept2 -> node);
	qsort(IntNode[index] -> node, strlen(IntNode[index] -> node), sizeof(char), comp_char);
      }
    }
    //Beschriftung des neu entstandenen Knoten
    strcpy(NODEPT2 -> node, nodept1 -> node);
    strcat(NODEPT2 -> node, nodept2 -> node);
    NODEPT2 -> geburt = time;
    
    qsort(NODEPT2 -> node, strlen(NODEPT2 -> node), sizeof(char), comp_char);
    //printf("  neu:%s\n", NODEPT2 -> node);//------------------------------------------------------------------------------<DEBUG
  }
  sTree::StringSort();
  return rueckgabe;

}


//Diese Funktion gibt den Index des Elternknotens zum uebergebenen Knotenindex
//Elternknoten vom Wurzelknoten ist der Wurzelknoten selbst
struct IntNode * sTree::ElternBestimmung(struct IntNode * nodept){
  int laenge;
  struct IntNode * candidate;
  int i;
  laenge = strlen(IntNode[0] -> node); 		   //Initialisiere Laenge mit Wurzelknotenlaenge

  if( (int) strlen(nodept -> node) == laenge){     //Eingabeknoten ist Wurzelknoten
    return IntNode[0];
  }
  candidate = IntNode[0];
  for(i = 1; i < n; i++){
    if(enthalten_in(nodept -> node, IntNode[i] -> node) == 1 && (int) strlen(nodept -> node) < laenge ){
      candidate = IntNode[i];
      laenge = strlen(IntNode[i] -> node);
    }
  }
  return candidate;
}


//Diese Funktion beschreibt die Intervallliste mit den Zeiten und den entsprechenden Kanten
void sTree::WriteIntervallData(){
  int i;
  int j;
  int k;
  //Zunaechst sortieren wir die Baumknoten in zeitlicher Abfolge
  sTree::AgeSort();
  
  intervall[0] -> start = 0;
  intervall[0] -> stop = 0;
  intervall[0] -> kanten = NULL;

  L = 0;
  for(i = 1; i<= b; i++){	//b ist die anzahl der blaetter in stree
    intervall[i] -> start = intervall[i - 1] -> stop;
    intervall[i] -> stop = IntNode[i - 1] -> tod;
    intervall[i] -> laenge = intervall[i] -> stop - intervall[i] -> start;
    //printf("%f\n", intervall[i] -> laenge);
    L = L + i * (intervall[i] -> stop - intervall[i] -> start); 
    //printf("%f  ", L);
    k = 0;
    if(intervall[i] -> start < intervall[i] -> stop){
	for(j = 0; j < n; j++){
	    if(IntNode[j] -> tod >= intervall[i] -> stop && sTree::ElternBestimmung(IntNode[j]) -> tod <= intervall[i] -> start){
		intervall[i] -> kanten[k] = IntNode[j];
		k++;
	    }
	}
    }
    else{
      intervall[i] -> kanten = NULL;
    }
  }
  sTree::StringSort();
}


//Diese Funktion gibt einen Zeiger auf die Bruderkante zum uebergebenenKnoten (~ Kante). Bruderkante = Kante mit gleichen Eltern Bruder(Wurzelkante) := Wurzelkante
struct IntNode *  sTree::BruderBestimmung(struct IntNode * nodept){
   struct IntNode * NODEPT;
   int i;
   NODEPT = sTree::ElternBestimmung(nodept);
   if(NODEPT == nodept){
     return nodept;
   }
   i = 1;
   while( i < n && !(NODEPT == sTree::ElternBestimmung(IntNode[i]) && nodept != IntNode[i])){
     i++;
   }
   if(i == n){
     return nodept;
   }else{
     return IntNode[i];
   }
}

//Diese Funktion bekommt zwei Zeiger auf Baumknoten und berechnet die Laenge des Durchschnittes der Lebensintervalle
double sTree::GemeinsameLebenszeit(struct IntNode * p1, struct IntNode * p2){

  if(p2 -> geburt >= p1 -> geburt ){
    return p1 -> tod - p2 -> geburt;
  }else{
    return p2 -> tod - p1 -> geburt;
  }
}

//Diese Funktion bekommt Zeiger auf zwei Knoten und gibt das das gemeinsame Lebensintervall zurueck
struct LebensIntervall sTree::GemeinsamesLebensintervall(struct IntNode * p1, struct  IntNode * p2){
  struct LebensIntervall intervall;
  if(p1 -> geburt <= p2 -> geburt && p1 -> tod >= p2 -> geburt){
    intervall.von = p2 -> geburt;

    intervall.bis = ( p1 -> tod <= p2 -> tod) ? p1 -> tod : p2 -> tod;

  }else if(p1 -> geburt >= p2 -> geburt && p1 -> geburt <= p2 -> tod){

    intervall.von = p1 -> geburt;

    intervall.bis = (p1 -> tod <= p2 -> tod) ? p1 -> tod : p2 -> tod;

  }else{
    intervall.von = -1;
    intervall.bis = -1;
  }
  return intervall;
}

//Diese Funktion berechnet die Wahrscheinlichkeit, dass zwischen den angegebenen Kanten horizontaler Gentransfer stattfindet
double sTree::BerechneWkeit(struct IntNode * p1, struct IntNode * p2){
  double w_keit;
  struct LebensIntervall cur_intervall;
  int j;
  cur_intervall = sTree::GemeinsamesLebensintervall(p1, p2);
  w_keit = 0;
  if(cur_intervall.von == cur_intervall.bis){
    return w_keit;
  }
  j = 2;
  while(intervall[j] -> start < cur_intervall.von){
    j++;
  }
  while(j <= b && intervall[j] -> stop <= cur_intervall.bis){
    w_keit =  w_keit + ((intervall[j] -> laenge) / ( L * (j - 1)) );
    j++;
  }
  return w_keit;
}

//Diese Funktion schreibt die Baumtopologie (sortiert als Sring) auf die Ausgabe
void sTree::PrintTopologie(){
  int i;

  sTree::StringSort();
  //printf("%s", IntNode[0] -> node);
  for(i = 0; i < n; i++){
    if(IntNode[i] -> node){
      printf(" %s", IntNode[i] -> node);   
    }
    else{
      printf(" -");  
    }
  }
}

//Diese Funktion bekommt eine Zeit und gibt die Nr. des Intervalles zurueck in das diese Zeit faellt
int sTree::BestimmeIntervall(double time){
   int i = 0;
   while(time > intervall[i] -> stop || intervall[i] -> start == intervall[i] -> stop){
     i++;
   }
   return i;
}

//Diese methode gibt das Alter des Baumes an
double sTree::Baumalter(){
   int i;
   double alter = 0;
   for(i = 0; i < n; i++){
      alter = (alter > (IntNode[i] -> tod)) ? alter : IntNode[i] -> tod;
   }
   return alter;
}


//bestimmt die poissonvariable bei baeumen mit unterschiedlichen hgt raten an den aesten (summe(rate(i)*laenge(i))) (fuer hgt_var_simrate)
//wenn alle hgt raten 0 im baum sind --> poissonvar=0 --> es finden 0 umhaengungen statt --> baumtopologie bleibt konstant
/////////////////////////////////////////////////NEU
double sTree::bestimmePoissonvar(){
   double poissonvar = 0;
   int i;
   for(i = 0; i < n; i++){
     poissonvar=poissonvar+IntNode[i] -> rate * (IntNode[i] -> tod - IntNode[i] -> geburt);    
   }
   return poissonvar;
}

/////////////////////////////////////////////////NEU
//diese Funktion bekommt die Liste mit den Umhaengedaten und traegt zu jeder zeit im ratenbaumgroessenintervall das baumgroessenintervall ein (fuer hgt_var_simrate)
void sTree::ZeitImBaumgroessenintervall(struct umhaengedaten ** liste, int anzahl){
  int i;
  int j;
  int kontrolle=0;
  double linke_ratenbaumintervallgrenze;
  double linke_baumintervallgrenze;
  
  for(i = 0; i < n; i++){  //ueber alle knoten
     if (i==0){
       linke_ratenbaumintervallgrenze = 0;
       linke_baumintervallgrenze = 0;
     } 
     else{
       linke_ratenbaumintervallgrenze = IntNode[i-1] -> rechts_ratenbaumgroessenintervall;
       linke_baumintervallgrenze = IntNode[i-1] -> rechts_baumgroessenintervall;
     }  
     for(j = 0; j < anzahl; j++){  //ueber alle umhaengungen
       if ((liste[j] -> zeit_im_raten_baumgroessenintervall > linke_ratenbaumintervallgrenze)&&(liste[j] -> zeit_im_raten_baumgroessenintervall <= IntNode[i] -> rechts_ratenbaumgroessenintervall)){
         kontrolle++;
	 double diff=liste[j] -> zeit_im_raten_baumgroessenintervall - linke_ratenbaumintervallgrenze;
	 double neuer_intervall_abschnitt=diff / IntNode[i] -> rate;    //ausnahmefall fuer rate=0 --> division durch 0? nein, da in einems solchen intervall kein hgt event stattfindet
	 liste[j] -> zeit_im_baumgroessenintervall = linke_baumintervallgrenze + neuer_intervall_abschnitt;
	 //printf("zeit im baumgroessenintervall %f\tzeit im ratenbaumgroessenintervall %f\t%s Knoten\n", liste[j] -> zeit_im_baumgroessenintervall, liste[j] -> zeit_im_raten_baumgroessenintervall, IntNode[i] -> node);
	 //printf("%s Knoten\n", IntNode[i] -> node);
       }
     }
   }
   if (kontrolle !=anzahl){
     printf("Es konnten nicht alle Umhaengungen korrekt zugeordnet werden.\n");
     exit(1);
   }
}

//Diese Funktion bekommt die Liste mit den Umhaengezeiten und traegt
//die fehlenden Daten (Kanten, Zeitpunkt) ein.
int sTree::VervollstaendigeUmhaengedaten(struct umhaengedaten ** liste, int anzahl){		  
			
     int i;
     int j;
     int k;
     int l;
     double zeitvariable;
     
     l = 0;
     for(j = 0; j < anzahl; j++){
       i = 0;
       zeitvariable = 0;
       while(zeitvariable < liste[j] -> zeit_im_baumgroessenintervall){
       i++;
       zeitvariable = zeitvariable + IntNode[i] -> tod - IntNode[i] -> geburt;
     }
     liste[j] -> kante = i;
     liste[j] -> zeitpunkt = IntNode[i] -> tod + liste[j] -> zeit_im_baumgroessenintervall - zeitvariable;
     liste[j] -> intervall_nr = BestimmeIntervall(liste[j] -> zeitpunkt);

     if(liste[j] -> zeitpunkt == intervall[liste[j] -> intervall_nr] -> start){  //wenn zeitpunkt gleich zeitpunkt der intervallgrenze ist
       ++l;
       liste[j] -> zeitpunkt +=  l * (intervall[liste[j] -> intervall_nr] -> laenge / (anzahl + 1));
     }
     k = 0;
     while((IntNode[i] !=  intervall[liste[j] -> intervall_nr] -> kanten[k]) &&
           (k < liste[j] -> intervall_nr) ){
       k++;
     }
     if(k == liste[j] -> intervall_nr){
       printf("IntervallNr=%i Kante=%s\n", liste[j] -> intervall_nr, intervall[liste[j] -> intervall_nr] -> kanten[k] -> node);
       printf("Fehler beim Vervollstaendigen der Umhaengedaten: stringtree.cpp\n");
       exit(3);
     }
     else{
       liste[j] -> lfd_nr_im_intervall = k;
     }
   }
   return l;
}



// ENDE DER METHODEN ================================================
//===================================================================



//===================================================================
//SONSTIGE FUNKTIONEN

//Vergleichsfunktionen fuer die Qsort-Aufrufe

//Fuer die Sortierung der Klasse sTree nach Alter
int comp_age(const void * s1, const void * s2){
   struct IntNode * pt1;
   struct IntNode * pt2;
   double r;
   pt1 = *(struct IntNode **) s1;
   pt2 = *(struct IntNode **) s2;

   r = pt1 -> tod - pt2 -> tod;
   if(r < 0){
     return -1;
   }
   else if(r == 0){
     return 0;
   }
   else{
     return 1;
   }
}

// Ein String wird alphabetisch umsortiert
int comp_char(const void * s1, const void * s2){
   int a;
   int b;
   a = *(char *) s1 - '0';
   b = *(char *) s2 - '0';
   return a - b;
}

//Fuer die Sortierung der Klasse sTree nach Bezeichnerlaenge und Alphabet
int comp_strings(const void * s1, const void * s2){
  struct IntNode * pt1;
  struct IntNode * pt2;
  pt1 = *(struct IntNode **) s1;
  pt2 = *(struct IntNode **) s2;
  if(strlen(pt1 -> node) > strlen(pt2 -> node)){
    return -1;
  }
  else if(strlen(pt1 -> node) == strlen(pt2 -> node)){
    return strcmp(pt1 -> node, pt2 -> node);
  }
  else{
    return 1;
  }
}

//Funktion die testet ob ein String (byteweise) in einem anderen enthalten ist
//Bsp: xyz ist in axfghtybbz enthalten
//Ruekgabe +1 (enthalten)
//         -1 (nicht enthalten)
//          0 (beide Knoten identisch)
int enthalten_in(char * substring, char * string){
   int laenge_substring;
   int laenge_string;
   int i;
   char * pt;
   laenge_substring = strlen(substring);
   laenge_string = strlen(string);

   i = 0;
   pt = string;
   while(i < laenge_substring && (pt = strchr(pt, substring[i])) != NULL){
     i++;
   }
   if(i == laenge_substring){

     if( strcmp(string, substring) == 0){
       return 0;
     }
     else{
       return 1;
     }
   }
   else{
     return -1;
   }
}

//Diese Funkktion erhaelt zwei String und entfernt alle Zeichen aus dem zweiten, die im ersten enthalten sind
//Der zweite String wird dann durch den resultierenden ersetzt
void entferne_substring(char * substring, char * string){
  int i;
  int j;
  i = 0;
  j = 0;
  while(string[j] != '\0'){
    if(strchr(substring, string[j]) == NULL){
      string[i] = string[j];
      i++;
      j++;
    }
    else{
      j++;
    }
  }
  string[i] = '\0';
}


